@@ -47,16 +47,17 @@ public:
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
 
-    void startConnection() { Start(); };
-    void stopConnection() { Stop(); };
+    void startConnection() { isRunning_m = true; Start(); };
+    void stopConnection() { isRunning_m = false; Stop(); };
 
     void addTelegramListener(TelegramListener *listener);
     bool removeTelegramListener(TelegramListener *listener);
     void write(eibaddr_t gad, uint8_t* buf, int len);
-    bool checkInput();
+    int checkInput();
 
 private:
     EIBConnection *con_m;
+    bool isRunning_m;
     pth_event_t stop_m;
     std::string url_m;
     TelegramListener *listener_m;
