@@ -41,6 +41,10 @@ Object* Object::create(const std::string& type)
     return new ValueObject();
   else if (type == "EIS6")
     return new ScalingObject();
+  else if (type == "EIS4")
+    return new DateObject();
+  else if (type == "EIS3")
+    return new TimeObject();
   else if (type == "EIS2")
     return new DimmingObject();
   else
@@ -63,13 +67,17 @@ Object* Object::create(ticpp::Element* pConfig)
 
 void Object::importXml(ticpp::Element* pConfig)
 {
-  id_m = pConfig->GetAttribute("id");
-  std::string gad = pConfig->GetAttribute("gad");
-  if (id_m == "")
+  std::string id = pConfig->GetAttribute("id");
+  if (id == "")
     throw ticpp::Exception("Missing or empty object ID");
-  ObjectController::instance()->addObject(this);
+  if (id_m == "")
+    id_m = id;
 
-  if (gad != "")
+  std::string gad = pConfig->GetAttributeOrDefault("gad", "nochange");
+    // set default value to "nochange" just to see if the attribute was present or not in xml
+  if (gad == "")
+    gad_m = 0;
+  else if (gad != "nochange")
     gad_m = readgaddr(gad.c_str());
 
   forcewrite_m = (pConfig->GetAttribute("forcewrite") == "true");
@@ -109,6 +117,52 @@ void Object::importXml(ticpp::Element* pConfig)
   std::cout << "Configured object '" << id_m << "': gad='" << gad_m << "'" << std::endl;
 }
 
+void Object::updateXml(ticpp::Element* pConfig)
+{
+  std::string gad = pConfig->GetAttributeOrDefault("gad", "nochange");
+    // set default value to "nochange" just to see if the attribute was present or not in xml
+  if (gad == "")
+    gad_m = 0;
+  else if (gad != "nochange")
+    gad_m = readgaddr(gad.c_str());
+
+  forcewrite_m = (pConfig->GetAttribute("forcewrite") == "true");
+
+  initValue_m = pConfig->GetAttribute("init");
+  if (initValue_m != "" && initValue_m != "request") {
+		std::istringstream val(initValue_m);
+		float init_value;
+		val >> init_value;
+	
+		if ( val.fail() )
+		{
+			if (initValue_m == "persist")
+				init_value = PersistentStorage::read(id_m);
+			else if (initValue_m == "on" || initValue_m == "true" || initValue_m == "comfort")
+				init_value = 1;
+			else if (initValue_m == "off" || initValue_m == "false")
+				init_value = 0;
+			else if (initValue_m == "standby")
+				init_value = 2;
+			else if (initValue_m == "night")
+				init_value = 3;
+			else if (initValue_m == "frost")
+				init_value = 4;
+			else
+			{
+				std::stringstream msg;
+				msg << "Object: Bad init value: '" << initValue_m << "'" << std::endl;
+				throw ticpp::Exception(msg.str());
+			}
+		}
+    std::cout << "Object setting initial value to '" << init_value << "'" << std::endl;
+    setFloatValue(init_value);
+  }
+
+  descr_m = pConfig->GetText(false);
+  std::cout << "Reconfigured object '" << id_m << "': gad='" << gad_m << "'" << std::endl;
+}
+
 void Object::exportXml(ticpp::Element* pConfig)
 {
     pConfig->SetAttribute("id", id_m);
@@ -201,16 +255,6 @@ void SwitchingObject::onWrite(const uint8_t* buf, int len)
     }
 }
 
-void SwitchingObject::setIntValue(int value)
-{
-    setBoolValue(value != 0);
-}
-
-void SwitchingObject::setFloatValue(float value)
-{
-    setBoolValue(value != 0);
-}
-
 void SwitchingObject::setBoolValue(bool value)
 {
     if (!init_m || value != value_m || forcewrite_m)
@@ -270,14 +314,202 @@ void DimmingObject::setIntValue(int value)
     }
 }
 
-void DimmingObject::setFloatValue(float value)
+TimeObject::TimeObject() : wday_m(0), hour_m(0), min_m(0), sec_m(0)
 {
-    setIntValue((int)value);
 }
 
-void DimmingObject::setBoolValue(bool value)
+TimeObject::~TimeObject()
 {
-    setFloatValue(value ? 1 : 0);
+}
+
+void TimeObject::exportXml(ticpp::Element* pConfig)
+{
+    Object::exportXml(pConfig);
+    pConfig->SetAttribute("type", "EIS3");
+}
+
+void TimeObject::onWrite(const uint8_t* buf, int len)
+{
+    if (len < 5)
+    {
+        std::cout << "Invlalid packet received for TimeObject (too short)" << std::endl;
+        return;
+    }
+    int wday, hour, min, sec;
+    Object::onWrite(buf, len);
+
+    wday = (buf[2] & 0xE0) >> 5;
+    hour = buf[2] & 0x1F;
+    min = buf[3];
+    sec = buf[4];
+    if (!init_m || wday != wday_m || hour != hour_m || min != min_m || sec != sec_m)
+    {
+        std::cout << "New value " << wday << " " << hour << ":" << min << ":" << sec << " for time object " << getID() << std::endl;
+        wday_m = wday;
+        hour_m = hour;
+        min_m = min;
+        sec_m = sec;
+        init_m = true;
+        onUpdate();
+    }
+}
+
+void TimeObject::setIntValue(int value)
+{
+    if (value < 0)
+        setTime(time(0));
+    else
+    {
+        int rem = value;
+        int sec = rem % 60;
+        rem = (rem-sec) / 60; 
+        int min = rem % 60;
+        rem = (rem-min) / 60; 
+        int hour = rem % 24;
+        rem = (rem-hour) / 24; 
+        int wday = rem % 8;
+
+        setTime(wday, hour, min, sec);
+    }
+}
+
+int TimeObject::getIntValue()
+{
+    if (!init_m)
+        read();
+    return ((wday_m*24 + hour_m) * 60 + min_m) * 60 + sec_m;
+}
+
+void TimeObject::setTime(time_t time)
+{
+    struct tm * timeinfo = localtime(&time);
+    int wday = timeinfo->tm_wday;
+    if (wday == 0)
+        wday = 7;
+		setTime(wday, timeinfo->tm_hour, timeinfo->tm_min, timeinfo->tm_sec);
+}
+
+void TimeObject::setTime(int wday, int hour, int min, int sec)
+{
+    if (!init_m ||
+        wday_m != wday ||
+        hour_m != hour ||
+        min_m != min ||
+        sec_m != sec ||
+        forcewrite_m)
+    {
+        std::cout << "TimeObject: setTime "
+						    	<< wday << " "
+							    << hour << ":"
+							    << min << ":"
+							    << sec << std::endl;
+        wday_m = wday;
+        hour_m = hour;
+        min_m = min;
+        sec_m = sec;
+
+        uint8_t buf[5] = { 0, 0x80, ((wday<<5) & 0xE0) | (hour & 0x1F), min, sec };
+        
+        Services::instance()->getKnxConnection()->write(getGad(), buf, 5);
+        init_m = true;
+        onUpdate();
+    }
+}
+
+DateObject::DateObject() : day_m(0), month_m(0), year_m(0)
+{
+}
+
+DateObject::~DateObject()
+{
+}
+
+void DateObject::exportXml(ticpp::Element* pConfig)
+{
+    Object::exportXml(pConfig);
+    pConfig->SetAttribute("type", "EIS4");
+}
+
+void DateObject::onWrite(const uint8_t* buf, int len)
+{
+    if (len < 5)
+    {
+        std::cout << "Invlalid packet received for DateObject (too short)" << std::endl;
+        return;
+    }
+    int day, month, year;
+    Object::onWrite(buf, len);
+
+    day = buf[2];
+    month = buf[3];
+    year = buf[4];
+    if (year < 90)
+        year += 100;
+    if (!init_m || day != day_m || month != month_m || year != year_m)
+    {
+        std::cout << "New value " << year+1900 << "-" << month << "-" << day << " for date object " << getID() << std::endl;
+        day_m = day;
+        month_m = month;
+        year_m = year;
+        init_m = true;
+        onUpdate();
+    }
+}
+
+void DateObject::setIntValue(int value)
+{
+    if (value < 0)
+        setDate(time(0));
+    else
+    {
+        int rem = value;
+        int day = rem % 32;
+        rem = (rem-day) / 32; 
+        int month = rem % 13;
+        rem = (rem-month) / 13; 
+        int year = rem % 256;
+
+        setDate(day, month, year);
+    }
+}
+
+int DateObject::getIntValue()
+{
+    if (!init_m)
+        read();
+    return (year_m*13 + month_m) * 32 + day_m;
+}
+
+void DateObject::setDate(time_t time)
+{
+    struct tm * timeinfo = localtime(&time);
+		setDate(timeinfo->tm_mday, timeinfo->tm_mon+1, timeinfo->tm_year);
+}
+
+void DateObject::setDate(int day, int month, int year)
+{
+    if (year >= 1900)
+        year -= 1900;
+    if (!init_m ||
+        day_m != day ||
+        month_m != month ||
+        year_m != year ||
+        forcewrite_m)
+    {
+        std::cout << "DateObject: setDate "
+						    	<< year + 1900 << "-"
+							    << month << "-"
+							    << day << std::endl;
+        day_m = day;
+        month_m = month;
+        year_m = year;
+
+        uint8_t buf[5] = { 0, 0x80, day, month, year };
+        
+        Services::instance()->getKnxConnection()->write(getGad(), buf, 5);
+        init_m = true;
+        onUpdate();
+    }
 }
 
 ValueObject::ValueObject() : value_m(0)
@@ -319,11 +551,6 @@ void ValueObject::onWrite(const uint8_t* buf, int len)
     }
 }
 
-void ValueObject::setIntValue(int value)
-{
-    setFloatValue((float)value);
-}
-
 void ValueObject::setFloatValue(float value)
 {
     if (!init_m || value != value_m || forcewrite_m)
@@ -355,17 +582,11 @@ void ValueObject::setFloatValue(float value)
         buf[3] = (m & 0xff);
         
         Services::instance()->getKnxConnection()->write(getGad(), buf, 4);
+        init_m = true;
         onUpdate();
     }
 }
 
-void ValueObject::setBoolValue(bool value)
-{
-    setFloatValue(value ? 1 : 0);
-}
-
-
-
 ScalingObject::ScalingObject() : value_m(0)
 {
 }
@@ -406,21 +627,11 @@ void ScalingObject::setIntValue(int value)
         buf[2] = (value & 0xff);
         
         Services::instance()->getKnxConnection()->write(getGad(), buf, 3);
+        init_m = true;
         onUpdate();
     }
 }
 
-void ScalingObject::setFloatValue(float value)
-{
-    setIntValue((int)value);
-}
-
-void ScalingObject::setBoolValue(bool value)
-{
-    setIntValue(value ? 1 : 0);
-}
-
-
 ObjectController::ObjectController()
 {
     Services::instance()->getKnxConnection()->addTelegramListener(this);
@@ -472,16 +683,47 @@ void ObjectController::addObject(Object* object)
 
 void ObjectController::importXml(ticpp::Element* pConfig)
 {
-  ticpp::Iterator< ticpp::Element > child;
-  for ( child = pConfig->FirstChildElement(); child != child.end(); child++ )
+  ticpp::Iterator< ticpp::Element > child("object");
+  for ( child = pConfig->FirstChildElement("object", false); child != child.end(); child++ )
   {
-    if (child->Value() == "object")
+    std::string id = child->GetAttribute("id");
+    bool del = child->GetAttribute("delete") == "true";
+    ObjectIdMap_t::iterator it = objectIdMap_m.find(id);
+    if (it == objectIdMap_m.end())
     {
+      if (del)
+        throw ticpp::Exception("Object not found");
       Object* object = Object::create(&(*child));
-      objectIdMap_m[object->getID()] = object;
-      eibaddr_t gad = object->getGad();
+      if (object->getGad() && !objectMap_m.insert(ObjectPair_t(object->getGad(), object)).second)
+      {
+        delete object;
+        throw ticpp::Exception("Object GAD is already registered");
+      }
+      objectIdMap_m.insert(ObjectIdPair_t(id, object));
+    }
+    else if (del)
+    {
+      eibaddr_t gad = it->second->getGad();
       if (gad)
-        objectMap_m[gad] = object;
+        objectMap_m.erase(gad);
+      delete it->second;
+      objectIdMap_m.erase(it);
+    }
+    else
+    {
+      eibaddr_t gad = it->second->getGad();
+      it->second->importXml(&(*child));
+      eibaddr_t gad2 = it->second->getGad();
+      if (gad != gad2)
+      {
+        if (gad2)
+        {
+          if (!objectMap_m.insert(ObjectPair_t(gad2, it->second)).second)
+            throw ticpp::Exception("New object GAD is already registered");
+        }
+        if (gad)
+          objectMap_m.erase(gad);
+      }
     }
   }
 
