@@ -45,6 +45,7 @@ public:
     static Object* create(const std::string& type);
 
     virtual void importXml(ticpp::Element* pConfig);
+    virtual void updateXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
 
     const char* getID() { return id_m.c_str(); };
@@ -56,11 +57,11 @@ public:
     void removeChangeListener(ChangeListener* listener);
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setIntValue(int value) {};
-    virtual void setFloatValue(float value) {};
-    virtual void setBoolValue(bool value) {};
-    virtual int getIntValue() { return getBoolValue() ? 1 : 0; };
-    virtual float getFloatValue() { return getBoolValue() ? 1 : 0; };
-    virtual bool getBoolValue() { return false; };
+    virtual void setFloatValue(float value) { setIntValue((int)value); };
+    virtual void setBoolValue(bool value) { setIntValue(value ? 1 : 0); };
+    virtual int getIntValue() { return 0; };
+    virtual float getFloatValue() { return getIntValue(); };
+    virtual bool getBoolValue() { return getIntValue() != 0; };
 protected:
     bool init_m;
     bool forcewrite_m;
@@ -81,10 +82,11 @@ public:
     virtual ~SwitchingObject();
 
     virtual void onWrite(const uint8_t* buf, int len);
-    virtual void setIntValue(int value);
-    virtual void setFloatValue(float value);
+    virtual void setIntValue(int value) { setBoolValue(value != 0); };
+    virtual void setFloatValue(float value) { setBoolValue(value != 0); };
     virtual void setBoolValue(bool value);
     virtual bool getBoolValue() { if (!init_m) read(); return value_m; };
+    virtual int getIntValue() { return getBoolValue() ? 1 : 0; };
 private:
     bool value_m;
 };
@@ -99,15 +101,50 @@ public:
 
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setIntValue(int value);
-    virtual void setFloatValue(float value);
-    virtual void setBoolValue(bool value);
     virtual int getIntValue() { if (!init_m) read(); return value_m; };
-    virtual float getFloatValue() { return getIntValue(); };
-    virtual bool getBoolValue() { return getIntValue() != 0; };
 private:
     int value_m;
 };
 
+class TimeObject : public Object
+{
+public:
+    TimeObject();
+    virtual ~TimeObject();
+
+    virtual void exportXml(ticpp::Element* pConfig);
+
+    virtual void onWrite(const uint8_t* buf, int len);
+    virtual void setIntValue(int value);
+    virtual void setTime(time_t time);
+    virtual void setTime(int wday, int hour, int min, int sec);
+    virtual int getIntValue();
+private:
+    int wday_m;
+    int hour_m;
+    int min_m;
+    int sec_m;
+};
+
+class DateObject : public Object
+{
+public:
+    DateObject();
+    virtual ~DateObject();
+
+    virtual void exportXml(ticpp::Element* pConfig);
+
+    virtual void onWrite(const uint8_t* buf, int len);
+    virtual void setIntValue(int value);
+    virtual void setDate(time_t time);
+    virtual void setDate(int day, int month, int year);
+    virtual int getIntValue();
+private:
+    int day_m;
+    int month_m;
+    int year_m;
+};
+
 class ValueObject : public Object
 {
 public:
@@ -117,9 +154,9 @@ public:
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
-    virtual void setIntValue(int value);
+    virtual void setIntValue(int value) { setFloatValue((float)value); };
     virtual void setFloatValue(float value);
-    virtual void setBoolValue(bool value);
+    virtual void setBoolValue(bool value) { setFloatValue(value ? 1 : 0); };
     virtual int getIntValue() { return (int)getFloatValue(); };
     virtual float getFloatValue() { if (!init_m) read(); return value_m; };
     virtual bool getBoolValue() { return getFloatValue() != 0; };
@@ -137,11 +174,7 @@ public:
 
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setIntValue(int value);
-    virtual void setFloatValue(float value);
-    virtual void setBoolValue(bool value);
     virtual int getIntValue() { if (!init_m) read(); return value_m; };
-    virtual float getFloatValue() { return getIntValue(); };
-    virtual bool getBoolValue() { return getIntValue() != 0; };
 private:
     int value_m;
 };
