@@ -30,9 +30,9 @@ RuleServer::RuleServer()
 
 RuleServer::~RuleServer()
 {
-  std::list<Rule*>::iterator it;
-  for (it = rulesList_m.begin(); it != rulesList_m.end(); it++)
-    delete (*it);
+  RuleIdMap_t::iterator it;
+  for (it = rulesMap_m.begin(); it != rulesMap_m.end(); it++)
+    delete (*it).second;
 }
 
 RuleServer* RuleServer::instance()
@@ -44,22 +44,37 @@ RuleServer* RuleServer::instance()
 
 void RuleServer::importXml(ticpp::Element* pConfig)
 {
-  ticpp::Iterator< ticpp::Element > child;
-  for ( child = pConfig->FirstChildElement(); child != child.end(); child++ )
+  ticpp::Iterator< ticpp::Element > child("rule");
+  for ( child = pConfig->FirstChildElement("rule", false); child != child.end(); child++ )
   {
-    Rule* rule = new Rule();
-    rule->importXml(&(*child));
-    rulesList_m.push_back(rule);
+    std::string id = child->GetAttribute("id");
+    bool del = child->GetAttribute("delete") == "true";
+    RuleIdMap_t::iterator it = rulesMap_m.find(id);
+    if (it == rulesMap_m.end())
+    {
+      if (del)
+        throw ticpp::Exception("Rule not found");
+      Rule* rule = new Rule();
+      rule->importXml(&(*child));
+      rulesMap_m.insert(RuleIdPair_t(id, rule));
+    }
+    else if (del)
+    {
+      delete it->second;
+      rulesMap_m.erase(it);
+    }
+    else
+      it->second->updateXml(&(*child));
   }
 }
 
 void RuleServer::exportXml(ticpp::Element* pConfig)
 {
-  std::list<Rule*>::iterator it;
-  for (it = rulesList_m.begin(); it != rulesList_m.end(); it++)
+  RuleIdMap_t::iterator it;
+  for (it = rulesMap_m.begin(); it != rulesMap_m.end(); it++)
   {
     ticpp::Element pElem("rule");
-    (*it)->exportXml(&pElem);
+    (*it).second->exportXml(&pElem);
     pConfig->LinkEndChild(&pElem);
   }
 }
@@ -99,7 +114,7 @@ void Rule::importXml(ticpp::Element* pConfig)
     bool isFalse = (*actionListIt).GetAttribute("type") == "on-false";
     std::cout << "ActionList: Configuring " << (isFalse ? "'on-false'" : "" ) << std::endl;
 		ticpp::Iterator<ticpp::Element> actionIt("action");
-		for (actionIt = (*actionListIt).FirstChildElement("action"); actionIt != actionIt.end(); actionIt++ )
+		for (actionIt = (*actionListIt).FirstChildElement("action", false); actionIt != actionIt.end(); actionIt++ )
 		{
 			Action* action = Action::create(&(*actionIt));
       if (isFalse)
@@ -111,6 +126,52 @@ void Rule::importXml(ticpp::Element* pConfig)
   std::cout << "Rule: Configuration done" << std::endl;
 }
 
+void Rule::updateXml(ticpp::Element* pConfig)
+{
+  std::string value = pConfig->GetAttribute("active");
+  if (value != "")
+    isActive_m = !(value == "off" || value == "false" || value == "no");
+
+  std::cout << "Rule: Reconfiguring " << getID() << " (active=" << isActive_m << ")" << std::endl;
+
+  ticpp::Element* pCondition = pConfig->FirstChildElement("condition", false);
+  if (pCondition != NULL)
+  {
+    if (condition_m != 0)
+      delete condition_m;
+    std::cout << "Rule: Reconfiguring condition " << getID() << std::endl;
+    condition_m = Condition::create(pCondition, this);
+  }
+
+  ticpp::Element* pActionList = pConfig->FirstChildElement("actionlist", false);
+
+  if (pActionList != NULL)
+  {
+    ActionsList_t::iterator it;
+    for(it=actionsList_m.begin(); it != actionsList_m.end(); ++it)
+      delete (*it);
+    for(it=actionsListFalse_m.begin(); it != actionsListFalse_m.end(); ++it)
+      delete (*it);
+
+    ticpp::Iterator<ticpp::Element> actionListIt("actionlist");
+    for ( actionListIt = pActionList; actionListIt != actionListIt.end(); actionListIt++ )
+    {
+      bool isFalse = (*actionListIt).GetAttribute("type") == "on-false";
+      std::cout << "ActionList: Reconfiguring " << (isFalse ? "'on-false'" : "" ) << std::endl;
+	  	ticpp::Iterator<ticpp::Element> actionIt("action");
+		  for (actionIt = (*actionListIt).FirstChildElement("action"); actionIt != actionIt.end(); actionIt++ )
+  		{
+	  		Action* action = Action::create(&(*actionIt));
+        if (isFalse)
+			    actionsListFalse_m.push_back(action);
+        else
+	  		  actionsList_m.push_back(action);
+		  }
+    }
+  }
+  std::cout << "Rule: Reconfiguration done" << std::endl;
+}
+
 void Rule::exportXml(ticpp::Element* pConfig)
 {
     if (id_m != "")
@@ -310,6 +371,8 @@ void SetValueAction::importXml(ticpp::Element* pConfig)
 						value_m = 3;
 				else if (value == "frost")
 						value_m = 4;
+				else if (value == "now")
+						value_m = -1;
         else
         {
             std::stringstream msg;
@@ -615,15 +678,29 @@ void TimerCondition::importXml(ticpp::Element* pConfig)
     else
         cl_m = 0;
 
-    ticpp::Element* at = pConfig->FirstChildElement("at");
-    at_m.importXml(at);
-    
+    ticpp::Element* at = pConfig->FirstChildElement("at", false);
+    ticpp::Element* every = pConfig->FirstChildElement("every", false);
+    if (at && every)
+        throw ticpp::Exception("Timer can't define <at> and <every> elements simultaneously");
+    if (at)
+        at_m.importXml(at);
+    else if (every)
+        every->GetText(&after_m);
+    else
+        throw ticpp::Exception("Timer must define <at> or <every> elements");
+
     ticpp::Element* during = pConfig->FirstChildElement("during", false);
     ticpp::Element* until = pConfig->FirstChildElement("until", false);
     if (during && until)
         throw ticpp::Exception("Timer can't define <until> and <during> elements simultaneously");
     if (during)
+    {
         during->GetText(&during_m);
+        if (every && after_m > during_m)
+            after_m -= during_m;
+        else if (every)
+            throw ticpp::Exception("Parameter <every> must be greater than <during>");
+    }
     else if (until)
     {
         until_m.importXml(until);
@@ -641,9 +718,21 @@ void TimerCondition::exportXml(ticpp::Element* pConfig)
     if (trigger_m)
         pConfig->SetAttribute("trigger", "true");
 
-    ticpp::Element pAt("at");
-    at_m.exportXml(&pAt);
-    pConfig->LinkEndChild(&pAt);
+    if (after_m == -1)
+    {
+        ticpp::Element pAt("at");
+        at_m.exportXml(&pAt);
+        pConfig->LinkEndChild(&pAt);
+    }
+    else
+    {
+        ticpp::Element pEvery("every");
+        int every = after_m;
+        if (during_m > 0)
+            every += during_m;
+        pEvery.SetText(every);
+        pConfig->LinkEndChild(&pEvery);
+    }
     
     if (during_m == -1)
     {
