@@ -191,6 +191,7 @@ public:
     virtual ~Rule();
 
     virtual void importXml(ticpp::Element* pConfig);
+    virtual void updateXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
 
     const char* getID() { return id_m.c_str(); };
@@ -220,7 +221,10 @@ public:
 private:
     RuleServer();
     ~RuleServer();
-    std::list<Rule*> rulesList_m;
+    typedef std::pair<std::string ,Rule*> RuleIdPair_t;
+    typedef std::map<std::string ,Rule*> RuleIdMap_t;
+    RuleIdMap_t rulesMap_m;
+//    std::list<Rule*> rulesList_m;
     static RuleServer* instance_m;
 };
 
