@@ -172,7 +172,7 @@ void TimeSpec::exportXml(ticpp::Element* pConfig)
 }
 
 PeriodicTask::PeriodicTask(ChangeListener* cl)
-    : cl_m(cl), nextExecTime_m(0), value_m(false), during_m(0)
+    : cl_m(cl), nextExecTime_m(0), value_m(false), during_m(0), after_m(-1)
 {
 }
 
@@ -195,21 +195,22 @@ void PeriodicTask::onTimer(time_t time)
   
 void PeriodicTask::reschedule(time_t now)
 {
-    TimeSpec* next;
     if (now == 0)
         now = time(0);
-    if (value_m && during_m > 0)
+    if (value_m)
     {
-        nextExecTime_m += during_m;
+        if (during_m != -1)
+            nextExecTime_m = now + during_m;
+        else
+            nextExecTime_m = findNext(now, &until_m);
     }
     else
     {
-				if (value_m && during_m == -1)
-						next = &until_m;
-				else
-						next = &at_m;
+        if (after_m != -1)
+            nextExecTime_m = now + after_m;
+        else
+            nextExecTime_m = findNext(now, &at_m);
 
-        nextExecTime_m = findNext(now, next);		
     }
     if (nextExecTime_m != 0) {
 				struct tm * timeinfo = localtime(&nextExecTime_m);
@@ -218,7 +219,8 @@ void PeriodicTask::reschedule(time_t now)
 									<< timeinfo->tm_mon + 1 << "-"
 									<< timeinfo->tm_mday << " "
 									<< timeinfo->tm_hour << ":"
-									<< timeinfo->tm_min << ":0 ("
+									<< timeinfo->tm_min << ":"
+									<< timeinfo->tm_sec << " ("
 									<< nextExecTime_m << ")" << std::endl;
 				Services::instance()->getTimerManager()->addTask(this);
     }
@@ -345,7 +347,7 @@ ExceptionDays::~ExceptionDays()
 void ExceptionDays::importXml(ticpp::Element* pConfig)
 {
   ticpp::Iterator< ticpp::Element > child;
-  for ( child = pConfig->FirstChildElement(); child != child.end(); child++ )
+  for ( child = pConfig->FirstChildElement(false); child != child.end(); child++ )
   {
     if (child->Value() == "date")
     {
