@@ -94,7 +94,7 @@ public:
 
 protected:
     TimeSpec at_m, until_m;
-    int during_m;
+    int during_m, after_m;
     time_t nextExecTime_m;
     ChangeListener* cl_m;
     bool value_m;
