@@ -862,7 +862,7 @@ namespace ticpp
 		@endcode
 		@return Pointer the duplicate node.
 		*/
-		std::auto_ptr< Node > Clone() const;
+//		std::auto_ptr< Node > Clone() const;
 
 	protected:
 		/**
